function x = KalmanSmooth(yin,Model,p)
% Kalman Smoother  
% yin NxT
% Model structure of model parameters
% This program is based on Z. Ghahramani's LDS code
% This code is not optimized for speed
% Written by Emtiyaz @ IISc.

%reshape the data from NxT to 'h' for initial value and y
h=yin(:,1:p);%initial values KXN
ytemp=yin(:,p+1:end);
[N,T]=size(ytemp);
y=reshape(ytemp',T*N,1);
clear ytemp;

%subtract mean
N1=size(h,1);
htemp1=reshape(h,N1*p,1);
ytemp=[y' htemp1']';
Mu=mean(ytemp);
y=y-ones(N*T,1)*Mu;
h=h-ones(N1,p)*Mu;
clear htemp1 ytemp N1;

%bring the input data to the desired form
Y=reshape(y,T,N,1);
Y=permute(Y,[2 3 1]); 
%initialize the parameters
A=Model.A;
Q=Model.Q;
R=Model.R;
x0=Model.x0;
P0=Model.P0;

%get the estimate through kalman smoother
x=zeros(N,p,T);
for n=N:-1:1
	Y1=Y(n,:,:);
	h1=h(n,:)';
	Est=ksSQ(A,Q,R,x0,P0,h1,Y1);
	x(n,:,:)=squeeze(Est.x(1,1:p,:));
	clear Est Y1 h1;
end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  out=ksSQ(A,Q,R,x0,P0,h1,Y);
% Kalman smoother estimation equation 
% with square root filter to avoid round off error
%
% Square root filter equations are based on:
% Wolfgang Schneider. "systems of seemingly unrelated 
% regression equations with time varying coecients- 
% an interplay of Kalman filtering, scoring, EM- and minque-
% method". Computers Math. Applic., 24(8/9):1-16, 1992
%
% This code is not optimized for speed

out=struct('x',[]);

[N p T]=size(Y);
K=length(x0);
K1=size(h1,1);
tiny=exp(-700);
I=eye(K);
const=(2*pi)^(-p/2);
problem=0;
lik=0;
%initialization
Xpre=zeros(N,K);   % P(x_t | y_1 ... y_{t-1})
Xcur=zeros(N,K,T);   % P(x_t | y_1 ... y_t)
Xfin=zeros(N,K,T);   % P(x_t | y_1 ... y_T)    given all outputs

Ppre=zeros(K,K,T);
Ppret=zeros(K,K,T);
Pcur=zeros(K,K,T);
Pcurt=zeros(K,K,T);
Pfin=zeros(K,K,T); 
Pfint=zeros(K,K,T); 
Bt=zeros(K,K,T);

Pt=zeros(K,K); 
Pcov=zeros(K,K);
Kcur=zeros(K,p);
invP=zeros(p,p);
J=zeros(K,K,T);

B=[eye(K1);zeros(K-K1,K1)];

%%%%%%%%%%%%%%%
% FORWARD PASS

R=R+(R==0)*tiny;

Xpre=ones(N,1)*x0';
Ppre(:,:,1)=P0;
Ppret(:,:,1)=chol(P0)';
invR=diag(1./R);

ht=[h1;zeros(K-K1,1)];
htemp2=zeros(K,T+1);
htemp2(:,1)=ht;

Rt=chol(R)';
Qt=B*chol(Q)'*B';

for t=1:T
	Mat1in=zeros(2*K);
	Mat1out=zeros(2*K);
	Mat2in=zeros(K+1);
	Mat2out=zeros(K+1);

	temp1=diag(R)+ht'*Ppre(:,:,t)*ht;
	invP=inv(temp1);
	CP=ht*invP;
	Kcur=Ppre(:,:,t)*CP;
	KC=Kcur*ht';
	Ydiff=Y(:,:,t)-Xpre*ht;
	Xcur(:,:,t)=Xpre+Ydiff*Kcur'; 

	%initialize the matrix input matrix of square root filter
	Mat2in(1,1)=Rt';
	Mat2in(1,2:end)=zeros(1,K);
	Mat2in(2:end,1)=Ppret(:,:,t)'*ht;
	Mat2in(2:end,2:end)=Ppret(:,:,t)';
	% filter to get output matrix
	[Q1,Mat2out]=qr(Mat2in);
	% take output from the output matrix
	Pcurt(:,:,t)=Mat2out(2:end,2:end)';
	Pcur(:,:,t)=Pcurt(:,:,t)*Pcurt(:,:,t)';
	
	if (t<T)
		Xpre=Xcur(:,:,t)*A';
		%initialize the matrix input matrix of square root filter
		Mat1in(1:K,1:K)=Qt';
		Mat1in(1:K,K+1:end)=zeros(K);
		Mat1in(K+1:end,1:K)=Pcurt(:,:,t)'*A';
		Mat1in(K+1:end,K+1:end)=Pcurt(:,:,t)';
		%filter to get output matrix
		[Q1,Mat1out]=qr(Mat1in);
		%take output from the output matrix
		Ppret(:,:,t+1)=Mat1out(1:K,1:K)';
		Bt(:,:,t)=Mat1out(K+1:end,K+1:end)';
		Ppre(:,:,t+1)=Ppret(:,:,t+1)*Ppret(:,:,t+1)';
	end;

	% calculate likelihood
	detiP=sqrt(det(invP));
	if (isreal(detiP) & detiP>0)
	  lik=lik+N*log(detiP)-0.5*sum(sum(Ydiff.*(Ydiff*invP)));
	else
	  problem=1;
	end;
	%initialize ht for the next cycle
	if K~=1
	      ht=[Y(:,:,t) ht(1:K1-1)' zeros(1,K-K1)]';
	      htemp2(:,t+1)=ht;
	else
	      temp1=Y(:,:,t);
	      htemp2(:,t+1)=ht;
	end
end;  

lik=lik+N*T*log(const);

%%%%%%%%%%%%%%%
% BACKWARD PASS

A1=zeros(K);
A2=zeros(K);
A3=zeros(K);
Ptsum=zeros(K);
YX=zeros(p,K);
hPh=0;

  
t=T; 
ht=htemp2(:,t);

Xfin(:,:,t)=Xcur(:,:,t);
Pfin(:,:,t)=Pcur(:,:,t); 
Pfint(:,:,t)=Pcurt(:,:,t); 
Pt=Pfin(:,:,t) + Xfin(:,:,t)'*Xfin(:,:,t)/N; 
A2= -Pt;
Ptsum=Pt;

YX=(Y(:,:,t)'*Xfin(:,:,t))*ht;
hPh=ht'*Pt*ht;

for t=(T-1):-1:1
	Mat3in=zeros(2*K,K);
	Mat3out=zeros(2*K,K);
	%initialize th for this cycle
	ht=htemp2(:,t);
	J(:,:,t)=Pcur(:,:,t)*A'*inv(Ppre(:,:,t+1));
	Xfin(:,:,t)=Xcur(:,:,t)+(Xfin(:,:,t+1)-Xcur(:,:,t)*A')*J(:,:,t)';
	Pfin(:,:,t)=Pcur(:,:,t)+J(:,:,t)*(Pfin(:,:,t+1)-Ppre(:,:,t+1))*J(:,:,t)';
	Pt=Pfin(:,:,t) + Xfin(:,:,t)'*Xfin(:,:,t)/N; 
	Ptsum=Ptsum+Pt;
	YX=YX+(Y(:,:,t)'*Xfin(:,:,t))*ht;
	hPh=hPh+ht'*Pt*ht;
end;

A3= Ptsum-Pt;
A2= Ptsum+A2;

t=T;  
Pcov=(I-KC)*A*Pcur(:,:,t-1);
A1=A1+Pcov+Xfin(:,:,t)'*Xfin(:,:,t-1)/N;

for t=(T-1):-1:2
  Pcov=(Pcur(:,:,t)+J(:,:,t)*(Pcov-A*Pcur(:,:,t)))*J(:,:,t-1)';
  A1=A1+Pcov+Xfin(:,:,t)'*Xfin(:,:,t-1)/N;
end;    


if problem 
  fprintf(' problem  '); problem=0;
end;

if ( ~all(eig(Ppre(:,:,t+1))>0) | ~all(eig(Pcur(:,:,t))>0)),
disp('negative variances !');
end;

out.x=Xfin;
