function [ID,Dout]=data(i,N)
%generate the signal, Multimodal with time varying AR parameter
%Dindata --structure of the data parameters
%Doutdata -- Doutdata.y size(T*fs,N)
% Written by Emtiyaz @ IISc.

if i==1
	%data parameter for TVARslow
	ID.fs=128;        % sampling frequency
	ID.fx=0:.01:64;  % frequency steps for the spectrum
	ID.sigma_e=0.2;   % Noise variance
	ID.T=1024;         % Duration
	ID.N=N;           % number of realizations

        % Get the poles
	t=0:ID.T-1;
	r=0.03*sin(0.005*pi*t)+0.9;
	w=(pi/9)*cos(0.005*pi*t)+(pi/4);
	iota=(-1)^(.5);
	ID.z=exp(iota*w)*diag(r);
        % Generate TVAR 
	Dout=TVAR(ID);
        Dout.z = ID.z;

elseif i==2
	%data parameter for TVARfast
	ID.fs=128;
	ID.fx=0:.01:64;  % frequency steps for the spectrum
	ID.sigma_e=0.2;   % Noise variance
	ID.T=256;         % Duration
	ID.N=N;           % number of realizations
                          
        % Get the poles
	t=0:ID.T-1;       
	r=0.03*sin(0.02*pi*t)+0.9;
	w=(pi/9)*cos(0.02*pi*t)+(pi/4);
	iota=(-1)^(.5);
	ID.z=exp(iota*w)*diag(r);
        % Generate TVAR 
	Dout=TVAR(ID);
        Dout.z = ID.z;

elseif i==3
	%Linear chirp f(t) = a + b*t
	ID.fs=128;       %sampling frequency
	ID.fx=0:.001:64;  % frequency steps for the spectrum
	ID.a=0; ID.b=10; % starting and ending frequency for the linear chirp
	ID.sigma_e=1;    % noise variance
	ID.T=2*128;      % Time duration of one realization
	ID.N=N;          % number of realizations of signal
	Dout=lchirp(ID); 

elseif i==4
	%FM signal f(t) = f+ 0.1*pi*f*fmod*cos(2*pi*fmod*t)
	ID.fs=128;       %sampling frequency
	ID.fx=0:.001:64;  % frequency steps for the spectrum
	ID.f=19.2;        
	ID.fmod=1.28;    
	ID.T=2*128;      % Time duration of one realization
	ID.sigma_e=1.25; % noise variance
	ID.N=N;          % number of realizations of signal
	Dout=FM(ID);
else
	disp('error unknown option entered');
	Dout=0;
	ID=0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Doutdata=TVAR(Dindata)
%generate the signal, Multimodal with time varying AR parameter
%Dindata --structure of the data parameters
%Doutdata -- Doutdata.y size(T*fs,N)

%structure definition
Doutdata=struct('y',[],'a',[]);
%data parameters;
sigma_e=Dindata.sigma_e;
fs=Dindata.fs;
fx=Dindata.fx;
T=Dindata.T;
N=Dindata.N;
z=Dindata.z;
%p is the order of AR model
p=2;
%find AR coefficients
a=zeros(p,T);
H=zeros(T,size(fx,2));
%calculate spectrum
for t=T:-1:1
	temp=poly([z(t) conj(z(t))]);	
	a(:,t)=-temp(2:end)';
	H(t,:)=abs(freqz(1,[1 -a(:,t)'],fx,fs));
end

%generate N trials of the data
for n=1:N
	ytemp1=zeros(1,T);

	%generate noise
	e=sqrt(sigma_e).*randn(1,T);
	ytemp1(1:p)=sqrt(.00002)*randn(1,p);
	%generate signal
	for t=p+1:T;
		temp=ytemp1(t-p:t-1);
		temp1=fliplr(temp);
		temp2=temp1';
		ytemp1(t)=a(:,t)'*temp2+e(t);
	end
	y(n,:)=ytemp1;
end;

Doutdata.y=y;
Doutdata.a=a;
Doutdata.H=H;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Doutdata=lchirp(Dindata)
%generate the linear chirp signal
%f=a+b*t
%Dindata --structure of the data parameters
%Doutdata -- Doutdata.y size(N,T)

Doutdata=struct('y',[]);
%data parameters;
sigma_e=Dindata.sigma_e;
a=Dindata.a;
b=Dindata.b;
fs=Dindata.fs;
T=Dindata.T;
N=Dindata.N;

t=0:(1/fs):(T-1)/fs;
for n=1:N
	e=sqrt(sigma_e).*randn(1,size(t,2));
	f=zeros(size(t));
	for i=1:length(t)
		f(i)=a+b*t(i);
		ytemp1(i)=5*sin(pi*f(i)*t(i));
	end
	%add signal and noise
	ytemp2=(ytemp1+e)';
	y(n,:)=ytemp2';
end;
Doutdata.y=y;
Doutdata.ft=f;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Doutdata=FM(Dindata)
%generate the signal, single sinusoid with a phase 
%which is frequency modulated
%Dindata --structure of the data parameters
%Doutdata -- Doutdata.y size(T*fs,N)

%data parameters;
f=Dindata.f;
fmod=Dindata.fmod;
sigma_e=Dindata.sigma_e;
fs=Dindata.fs;
T=Dindata.T;
N=Dindata.N;

t=0:(1/fs):(T-1)/fs;
for n=1:N
	e=sqrt(sigma_e).*randn(1,size(t,2));
	ytemp1=5*sin(2*pi*f*(t+0.05*sin(2*pi*fmod*t)));
	ytemp2=(ytemp1+e)';
	y(n,:)=ytemp2';
end;
Doutdata.y=y;
ft=f+(0.1*pi*f*fmod*cos(2*pi*fmod*t));
Doutdata.ft=ft;
