function net= emtvar(yin,K,InitPar,cyc,tol);

% EM algorithm identification for time varying AR model
% This program is based on Z. Ghahramani's LDS code
% and provides an extension when h matrix is time varying and a vector 
% of the past measurements and x is a vector of AR parameters 
% Input Description
% X N*Tx1
% h Nxp
% K model order p
% T total time points
% InitPar Initial parameters (A,Q,R,x0,P0)
% Written by Emtiyaz @ IISc.

if nargin<5   tol=0.0001; end;
if nargin<4   cyc=100; end;
if nargin<3   T=N; end;
if nargin<2   K=2; end;
% net is  [A,C,Q,R,x0,P0,Mu,LL]
net=struct('type','lds','A',[],'Q',[],'R',[],'x0',[],'P0',[],'Mu',[],'LL',[]);

%reshape the data from NxT to 'h' for initial value and y
h=yin(:,1:K);%initial values KXN
ytemp=yin(:,K+1:end);
[N,T]=size(ytemp);
X=reshape(ytemp',T*N,1);
clear ytemp;

p=length(X(1,:));
N=length(X(:,1));
%subtract mean
N1=size(h,1);
htemp1=reshape(h,N1*K,1);
Xtemp=[X' htemp1']';
Mu=mean(Xtemp);
X=X-ones(N,1)*Mu;
h=h-ones(N1,K)*Mu;
clear htemp1 Xtemp N1;

%check the input data format (N should be divisible by T)
if (rem(N,T)~=0)
  disp('Error: Data matrix length must be multiple of sequence length T');
  return;
end;
N=N/T;

%bring the data in format
Y=reshape(X,T,N,p);
Y=permute(Y,[2 3 1]); % Y is (N,p,T), analogously to X

%refine initial estimate
Par = Kinit(Y,h,T,K,InitPar);
A = Par.A;
Q=Par.Q;
R=Par.R;
x0=Par.x0;
P0=Par.P0;

%initialize some quantity 
lik=0;
LL=[];
YY=sum(X.*X)'/(T*N); 
Xmean=zeros(N,K,T);
%EM iterations
count=0;
for cycle=1:cyc
	count=count+1;
 	
	%initialization for variables used in loop 
       Y1=zeros(1,p,T);
       h1=zeros(p,1);
       oldlik=lik;
       n_x0=zeros(K,1);
       n_P0=zeros(K);
       n_A1=zeros(K,K);
       n_A2=zeros(K);
       n_A3=zeros(K,K);
       n_YX=0;
       n_hPh=0;
       n_lik=0;
	%iterate for each observation
	for n=1:N
		%take the observation
		Y1=Y(n,:,:);
		h1=h(n,:)';
		% E STEP
		%[lik1,Xfin,Pfin,Ptsum,YX,hPh,A1,A2,A3]=kalmansmooth(A,Q,R,x0,P0,h1,Y1);
		[lik1,Xfin,Pfin,Ptsum,YX,hPh,A1,A2,A3]=ksSQlds(A,Q,R,x0,P0,h1,Y1);
		% Re-estimate new (n_)A,C,Q,R,x0,P0;
		x0i(n,:)=squeeze(Xfin(:,:,1));
		%Xmean(n,:,:)=squeeze(Xfin);
		n_x0=n_x0+Xfin(:,:,1)'; 
		n_P0=n_P0 + Pfin(:,:,1);
		n_A1=n_A1 + A1(1:K,1:K);
 		n_A2=n_A2 + A2;
		n_A3=n_A3 + A3(1:K,1:K);
		n_YX=n_YX + YX;
		n_hPh=n_hPh + hPh;
		n_lik=n_lik + lik1;
		clear lik1 Xfin Pfin Ptsum YX hPh A1 A2 A3;
	end 
	% M STEP
	x0=(1/N)*n_x0;
	T1=x0i-ones(N,1)*x0';
	P0=(n_P0 + T1'*T1)/N;

	R = YY-(2*n_YX'-n_hPh)/(N*T);
	A = (n_A1)*inv(n_A2);
	Q = (n_A3-A*(n_A1'))/(N*(T-1));

	%check for the round off errors
	if (det(Q)<0) 
	  fprintf('Q problem\n');
	end;
	%likelihood
	lik=n_lik;
	LL=[LL lik];
	fprintf('cycle %g lik %g',cycle,lik);
	%check if likelihood has not increased	
	if (cycle<=2)
	  likbase=lik;
	elseif (lik<oldlik) 
	  fprintf(' violation');
	elseif ((lik-likbase)<(1 + tol)*(oldlik-likbase)|~finite(lik)) 
	  fprintf('\n');
	  break;
	end;
	fprintf('\n');
	clear n_P0 n_A1 n_A2 n_A3 n_YX n_hPh n_lik x0i n_lik Y1 h1;
end;
%estimated parameters
net.A=A;
net.Q=Q;
net.R=R;
net.x0=x0;
net.P0=P0;
net.Mu=Mu;
net.LL=LL;
net.Xmean=Xmean;
net.count=count;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Kalman smoother estimation equation with square root filter for precision

function  [lik,Xfin,Pfin,Ptsum,YX,hPh,A1,A2,A3]=ksSQlds(A,Q,R,x0,P0,h1,Y);

out=struct('x',[]);
[N p T]=size(Y);
K=length(x0);
K1=size(h1,1);
tiny=exp(-700);
I=eye(K);
const=(2*pi)^(-p/2);
problem=0;
lik=0;
%initialization
Xpre=zeros(N,K);   % P(x_t | y_1 ... y_{t-1})
Xcur=zeros(N,K,T);   % P(x_t | y_1 ... y_t)
Xfin=zeros(N,K,T);   % P(x_t | y_1 ... y_T)    given all outputs

Ppre=zeros(K,K,T);
Ppret=zeros(K,K,T);
Pcur=zeros(K,K,T);
Pcurt=zeros(K,K,T);
Pfin=zeros(K,K,T); 
Pfint=zeros(K,K,T); 
Bt=zeros(K,K,T);

Pt=zeros(K,K); 
Pcov=zeros(K,K); 
Kcur=zeros(K,p);
invP=zeros(p,p);
J=zeros(K,K,T);

B=[eye(K1);zeros(K-K1,K1)];

%%%%%%%%%%%%%%%
% FORWARD PASS

R=R+(R==0)*tiny;

Xpre=ones(N,1)*x0';
Ppre(:,:,1)=P0;
Ppret(:,:,1)=chol(P0)';
invR=diag(1./R);

ht=[h1;zeros(K-K1,1)];
htemp2=zeros(K,T+1);
htemp2(:,1)=ht;

Rt=chol(R)';
Qt=B*chol(Q)'*B';

for t=1:T
	Mat1in=zeros(2*K);
	Mat1out=zeros(2*K);
	Mat2in=zeros(K+1);
	Mat2out=zeros(K+1);

	temp1=diag(R)+ht'*Ppre(:,:,t)*ht;
	invP=inv(temp1);
	CP=ht*invP;
	Kcur=Ppre(:,:,t)*CP;
	KC=Kcur*ht';
	Ydiff=Y(:,:,t)-Xpre*ht;
	Xcur(:,:,t)=Xpre+Ydiff*Kcur'; 
	%Pcur(:,:,t)=Ppre(:,:,t)-KC*Ppre(:,:,t);

	%initialize the matrix input matrix of square root filter
	Mat2in(1,1)=Rt';
	Mat2in(1,2:end)=zeros(1,K);
	Mat2in(2:end,1)=Ppret(:,:,t)'*ht;
	Mat2in(2:end,2:end)=Ppret(:,:,t)';
	%filter to get output matrix
	[Q1,Mat2out]=qr(Mat2in);
	%take output from the output matrix
	Pcurt(:,:,t)=Mat2out(2:end,2:end)';
	Pcur(:,:,t)=Pcurt(:,:,t)*Pcurt(:,:,t)';
	
	if (t<T)
		Xpre=Xcur(:,:,t)*A';
		%Ppre(:,:,t+1)=A*Pcur(:,:,t)*A'+Q;
		
		%initialize the matrix input matrix of square root filter
		Mat1in(1:K,1:K)=Qt';
		Mat1in(1:K,K+1:end)=zeros(K);
		Mat1in(K+1:end,1:K)=Pcurt(:,:,t)'*A';
		Mat1in(K+1:end,K+1:end)=Pcurt(:,:,t)';
		%filter to get output matrix
		[Q1,Mat1out]=qr(Mat1in);
		%take output from the output matrix
		Ppret(:,:,t+1)=Mat1out(1:K,1:K)';
		Bt(:,:,t)=Mat1out(K+1:end,K+1:end)';
		Ppre(:,:,t+1)=Ppret(:,:,t+1)*Ppret(:,:,t+1)';
	end;

	% calculate likelihood
	detiP=sqrt(det(invP));
	if (isreal(detiP) & detiP>0)
	  lik=lik+N*log(detiP)-0.5*sum(sum(Ydiff.*(Ydiff*invP)));
	else
	  problem=1;
	end;
	%initialize ht for the next cycle
	if K~=1
	      ht=[Y(:,:,t) ht(1:K1-1)' zeros(1,K-K1)]';
	      htemp2(:,t+1)=ht;
	else
	      temp1=Y(:,:,t);
	      htemp2(:,t+1)=ht;
	end
end;  


lik=lik+N*T*log(const);

%%%%%%%%%%%%%%%
% BACKWARD PASS

A1=zeros(K);
A2=zeros(K);
A3=zeros(K);
Ptsum=zeros(K);
YX=zeros(p,K);
hPh=0;

  
t=T; 
%ht=[ht(2:K)' Y(:,:,T-K)]';
ht=htemp2(:,t);

Xfin(:,:,t)=Xcur(:,:,t);
Pfin(:,:,t)=Pcur(:,:,t); 
Pfint(:,:,t)=Pcurt(:,:,t); 
Pt=Pfin(:,:,t) + Xfin(:,:,t)'*Xfin(:,:,t)/N; 
A2= -Pt;
Ptsum=Pt;

YX=(Y(:,:,t)'*Xfin(:,:,t))*ht;
hPh=ht'*Pt*ht;

for t=(T-1):-1:1
	Mat3in=zeros(2*K,K);
	Mat3out=zeros(2*K,K);
	%initialize th for this cycle
	ht=htemp2(:,t);
	J(:,:,t)=Pcur(:,:,t)*A'*inv(Ppre(:,:,t+1));
	Xfin(:,:,t)=Xcur(:,:,t)+(Xfin(:,:,t+1)-Xcur(:,:,t)*A')*J(:,:,t)';
	%Pfin(:,:,t)=Pcur(:,:,t)+J(:,:,t)*(Pfin(:,:,t+1)-Ppre(:,:,t+1))*J(:,:,t)';
	%initialize Mat3in
	Mat3in(1:K,:)=Pfint(:,:,t+1)'*J(:,:,t)';
	Mat3in(K+1:end,:)=Bt(:,:,t)';
	%calculate the variance estimate with square root filter 
	[Q1,Mat3out]=qr(Mat3in);
	Pfint(:,:,t)=Mat3out(1:K,:)';
	Pfin(:,:,t)=Pfint(:,:,t)*Pfint(:,:,t)';
	%sum the estimates for M-step
	Pt=Pfin(:,:,t) + Xfin(:,:,t)'*Xfin(:,:,t)/N; 
	Ptsum=Ptsum+Pt;
	YX=YX+(Y(:,:,t)'*Xfin(:,:,t))*ht;
	hPh=hPh+ht'*Pt*ht;
end;

A3= Ptsum-Pt;
A2= Ptsum+A2;

t=T;  
Pcov=(I-KC)*A*Pcur(:,:,t-1);
A1=A1+Pcov+Xfin(:,:,t)'*Xfin(:,:,t-1)/N;

for t=(T-1):-1:2
  Pcov=(Pcur(:,:,t)+J(:,:,t)*(Pcov-A*Pcur(:,:,t)))*J(:,:,t-1)';
  A1=A1+Pcov+Xfin(:,:,t)'*Xfin(:,:,t-1)/N;
end;    


if problem 
  fprintf(' problem  '); problem=0;
end;

if ( ~all(eig(Ppre(:,:,t+1))>0) | ~all(eig(Pcur(:,:,t))>0)),
disp('negative variances !');
end;

